/*----------------------------------------------------------------------------
 *      Name:    DEMO.C
 *      Purpose: USB HID Demo
 *      Version: V1.20
 *----------------------------------------------------------------------------
 *      This software is supplied "AS IS" without any warranties, express,
 *      implied or statutory, including but not limited to the implied
 *      warranties of fitness for purpose, satisfactory quality and
 *      noninfringement. Keil extends you a royalty-free right to reproduce
 *      and distribute executable files created using this software for use
 *      on NXP Semiconductors LPC family microcontroller devices only. Nothing 
 *      else gives you the right to use this software.
 *
 * Copyright (c) 2009 Keil - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/

#include "LPC177x_8x.h"                        /* LPC17xx definitions */

#include "lpc_types.h"

#include "lpc177x_8x_pinsel.h"
#include "lpc177x_8x_i2c.h"
#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"
#include "demo.h"
#include "joystick.h"
#include "pca9532.h"


#if POWERDOWN_MODE_USB_WAKEUP
#include "lpc177x_8x_timer.h"
extern volatile uint32_t SuspendFlag;
extern volatile uint32_t timer0_m0_counter;
extern volatile uint32_t USBActivityInterruptFlag;
volatile uint32_t WakeupFlag = 0;
#endif

uint8_t InReport;                              /* HID Input Report    */
                                            /*   Bit0   : Buttons  */
                                            /*   Bit1..7: Reserved */

uint8_t OutReport;                             /* HID Out Report      */
                                            /*   Bit0..7: LEDs     */


/*
 *  Get HID Input Report -> InReport
 */

void GetInReport (void) {
  uint8_t joy_val;

  joy_val = joystick_read();

  InReport = 0x00;
  if ((joy_val & JOYSTICK_UP)     != 0) InReport |= 0x01;  /* up     pressed means 0 */
  if ((joy_val & JOYSTICK_LEFT)   != 0) InReport |= 0x02;  /* left   pressed means 0 */
  if ((joy_val & JOYSTICK_RIGHT)  != 0) InReport |= 0x04;  /* right  pressed means 0 */
  if ((joy_val & JOYSTICK_CENTER) != 0) InReport |= 0x08;  /* select pressed means 0 */
  if ((joy_val & JOYSTICK_DOWN)   != 0) InReport |= 0x10;  /* down   pressed means 0 */
}


/*
 *  Set HID Output Report <- OutReport
 */

void SetOutReport (void) {
  uint16_t ledOnMask = 0;

  OutReport = OutReport;

  ledOnMask |= (OutReport << 8);
  
  pca9532_setLeds(0, 0xffff);
  pca9532_setLeds(ledOnMask, 0);

}

#if POWERDOWN_MODE_USB_WAKEUP
void TurnOffPLL0 (void)
{
  LPC_SC->PLL0CON   &= ~(0x1<<1);                /* PLL0 Disabled               */
  LPC_SC->PLL0FEED  = 0xAA;
  LPC_SC->PLL0FEED  = 0x55;
  while ( (LPC_SC->PLL0STAT & (1<<9)) != 0x00 ); /* Wait for PLOCK0 disconnect  */

  LPC_SC->PLL0CON   &= ~(0x1<<0);                /* PLL0 Disconnect             */
  LPC_SC->PLL0FEED  = 0xAA;
  LPC_SC->PLL0FEED  = 0x55;
  while ( (LPC_SC->PLL0STAT & (1<<8)) != 0x00 ); /* Wait for PLOCK0 shut down   */
  return;
}
#endif

/* Main Program */

int main (void) {
#if POWERDOWN_MODE_USB_WAKEUP
  uint32_t i, j;
#endif

//  SystemInit();

  LPC_IOCON->P1_25 |= 5; /* CLKOUT @ P1.25 */
  LPC_IOCON->P1_27 |= 4; /* CLKOUT @ P1.27 */


	// Config Pin for I2C_SDA and I2C_SCL of I2C0
	PINSEL_ConfigPin (0, 27, 1);
	PINSEL_ConfigPin (0, 28, 1);

  /* Initialize I2C */
  I2C_Init(I2C_0, 100000);
  I2C_Cmd(I2C_0, ENABLE);

  joystick_init();
  pca9532_init();

#if POWERDOWN_MODE_USB_WAKEUP
  /* TIME_INTERVAL is not used here. Using pre-defined 10MS timeval inside init_timer().
  will fix it later. */
  init_timer(0, 0, TIME_INTERVAL);		/* the timer is for USB suspend and resume */
#endif

  USB_Init();                           /* USB Initialization */
  USB_Connect(TRUE);                    /* USB Connect */

  while (1)                            /* Loop forever */
  {
#if POWERDOWN_MODE_USB_WAKEUP
	if ( USBActivityInterruptFlag )
	{
	  /* If the MCU is waked up from power down mode, the PLL needs to
	  be reconfigured, and USB block needs tn be reset and reconnect. */
	  USBActivityInterruptFlag = 0;
	  SystemInit();

	  init_timer(0, 0, TIME_INTERVAL);		/* the timer is for USB suspend and resume */

	  USB_Init();                           /* USB Initialization */
	  USB_Connect(TRUE);                    /* USB Connect */

	}
	/* bit 8 on USN_INT_STAT is USB NeedClk bit. */
	if ( SuspendFlag == 1 )
	{
	  if ( timer0_m0_counter > 200 )
	  {
		/* If it's a true suspend, turn off USB device bit the 
		USBNeed_clk will go away once USB device is disabled. */

		/* Below two lines are work-around that, in order to
		turn off the host clock, we enable HOST clock in USBClkCtrl
		first, suspend the host clock. Now, we can turn off the
		host clock in USBClkCtrl. If you force to turn off all
		clocks in USBClkCtrl, HOST clock bit will be set again
		inside USBClkSt. */
		LPC_USB->USBClkCtrl = 0x11;		/* Enable the AHB and USB host */
		LPC_USB->CommandStatus = 0x1;	/* Suspend the host */
		
		LPC_USB->USBClkCtrl = 0x00;		/* Turn off all the clocks */
		while ( LPC_USB->USBClkSt != 0x00 );	/* Wait for all clock off. */
		while ( LPC_SC->USBIntSt & (1 << 8) );	/* Wait for NeedClk go away. */
		/* Regular USB interrupt is disabled to test USBActivity interrupt. 
		It will be enabled once it's waken up. */
		NVIC_DisableIRQ(USB_IRQn);

		WakeupFlag = 1;
		for ( j = 0; j < 10; j++ )
	  	{
		  for ( i = 0; i < 0x200000; i++ );
		  LPC_GPIO2->SET = 0x000000F8;
		  for ( i = 0; i < 0x200000; i++ );
		  LPC_GPIO2->CLR = 0x000000F8;
	    }

		/* switch CCLK to system clock before turn off PLL0.  */
		LPC_SC->CCLKSEL  &= ~(0x3<<8);		
		TurnOffPLL0();
		 
		/* USB Activity interrupt won't occur until Deepsleep bit is set in SCR.
		UM needs to be updated regarding this. */
		SCB->SCR |= 0x04;	/* Set SLEEPDEEP bit in SCR in Cortex M3 core. */	
		LPC_SC->PCON = 0x1;
		__WFI();
	  }
	}
#endif

  }
}
